@@ -14,7 +14,7 @@ atan2(double y, double x)
   
   /* return a value between -PI and PI */
   if (x*x + y*y == 0) {
-    fprintf(stderr, "Sorry, can't request atan2(0, 0)\n");
+    fprintf(stderr, _("Sorry, the value atan2(0, 0) is indeterminate.\n"));
     exit(EXIT_FAILURE);
   }
   if (x == 0) return (y > 0) ? M_PI / 2: -M_PI / 2;
