@@ -11,7 +11,7 @@ pow(double x, double y)
   double result = 1.0, frac_y;
   
   if (x == 0 && y <= 0) {
-    fprintf(stderr, "Sorry, division by zero\n");
+    fprintf(stderr, _("Sorry, division by zero was encountered.\n"));
     exit(EXIT_FAILURE);
   }
   if (x == 0) return 0;
@@ -25,7 +25,7 @@ pow(double x, double y)
     if (x >= 0) result *= exp(frac_y * log(x));
     else {
       fprintf(stderr,
-	"Sorry, attempt to take fractional power of a negative number\n");
+	_("Sorry, fractional power of a negative number was attempted.\n"));
       exit(EXIT_FAILURE);
     }
   }
