@@ -349,7 +349,7 @@ unsigned int Star::Draw(const Rules &rules, StarViewer *sv,
 
   if (rules.StarLabels != NO_LABEL) { // number the star appropriately
     string starPlace;
-    label_t label = rules.StarLabels;
+    star_label_t label = rules.StarLabels;
 
     // make non-stellar object labels a different color
     sv->setcolor((sSpectrum.classmajor() == '*') ? NON_STELLAR_COLOR : 
