@@ -22,6 +22,7 @@
 //
 
 #include "stararray.h"
+#include "greek.h"
 
 #define NEED_FULL_NAMES
 #include "constellations.h"
@@ -217,16 +218,22 @@ void StarArray::Sort()
 //  its own Rules member.
 
 // Options:
-// If `casesensitive' is false, the function ignores the distinction
-//  between upper and lower case.  Otherwise all cases must match.
-// If `exactmatch' is true, `searchstring' must be identical to a star name
-//  (possibly excluding case) for a match.  If `exactmatch' is false, the
-//  function looks for `searchstring' as a substring of star names.  If, in
-//  addition, `searchstring' contains spaces, it is tokenized and all of the
-//  tokens must be substrings of the same star name for a match to occur.
-//  These substrings may not overlap.
-// If `exitonmatch' is true, the function returns only the first match found.
-//  Otherwise all matches are returned.
+//
+//  If `casesensitive' is false, the function ignores the distinction
+// between upper and lower case.  Otherwise all cases must match.
+//
+//  If `exactmatch' is true, `searchstring' must be identical to a star name
+// (possibly excluding case) for a match after some substitutions have been
+// performed.  If `exactmatch' is false, the function looks for `searchstring'
+// as a substring of star names.  If, in addition, `searchstring' contains
+// spaces, it is tokenized and all of the tokens must be substrings of the
+// same star name for a match to occur.  No substrings may overlap.
+//  In either case, some strings and characters are interpreted in special
+// ways.  See the StarPlot documentation, sections 3.2.1 and 4.3.2, for
+// details.
+//
+//  If `exitonmatch' is true, the function returns only the first match found.
+// Otherwise all matches are returned.
 
 void StarArray::Search(const std::string &search, StringList filelist, 
 		       const Rules &rules, bool casesensitive,
@@ -237,23 +244,81 @@ void StarArray::Search(const std::string &search, StringList filelist,
 
   string searchstring = search;
   starstrings::stripspace(searchstring);
-  if (! casesensitive) starstrings::toupper(searchstring);
+
+  // Interpret enclosing quote marks
+  unsigned size = searchstring.size();
+  if (size > 2 && searchstring[0] == '"' && searchstring[size - 1] == '"') {
+    searchstring = searchstring.substr(1, size - 2);
+    exactmatch = true;
+  }
+
+  // Replace real (UTF-8) lowercase Greek letters with their spelled-out names.
+  for (int i = 1; i < NUM_GREEK_LETTERS; i++) {
+    string name;
+    if (! exactmatch) name += '^';
+    name += Greek[i].name;
+ 
+    // Make a maximum of one replacement (it would make no sense for
+    // someone to include more than one Greek letter in a search string).
+    if (starstrings::find_and_replace(searchstring, Greek[i].utf8, name))
+      break;
+  }
   
+  if (! casesensitive) starstrings::toupper(searchstring);
+
   // Abbreviate constellation names in the search string, if any.
   for (int i = 0; i < NUM_CONSTELLATIONS; i++) {
-    string name = constelnames[i];
-    string abbr = constellations[i];
-    bool found = false;
+    string name(constelnames[i]);
+    string abbr;
+    if (!exactmatch) abbr += '[';
+    abbr += constellations[i];
+    if (!exactmatch) abbr += ']';
     if (! casesensitive) {
       starstrings::toupper(name);
       starstrings::toupper(abbr);
     }
+  
+    // Make a maximum of one replacement (it would make no sense for
+    // someone to include more than one constellation name in a search string).
+    if (starstrings::find_and_replace(searchstring, name, abbr))
+      break;
+  }
 
-    while ((found = starstrings::find_and_replace(searchstring, name, abbr)))
-      /* empty while() body */ ;
-    if (found) break;
+  // Replace asterisks with degree symbols.
+  while (starstrings::find_and_replace(searchstring, "*", DEGREE_UTF8))
+    /* empty body */ ;
+
+  const StringList temp_tokens(searchstring);
+  StringList start_tokens, end_tokens, tokens;
+
+  if (! exactmatch) {
+    int last = 0; // index of currently last element of tokens array
+    for (unsigned i = 0; i < temp_tokens.size(); i++, last++) {
+      unsigned s = temp_tokens[i].size();
+      if (s > 1 && temp_tokens[i][0] == '^') {
+	last--;
+	start_tokens.push_back(temp_tokens[i].substr(1, s - 1));
+      }
+      else if (s > 2 && temp_tokens[i][0] == '['
+		     && temp_tokens[i][s - 1] == ']') {
+	// Replace [ ] with spaces so that a search for "[token]" must turn
+	// up only instances where "token" appears surrounded by whitespace.
+	tokens.push_back(temp_tokens[i]);
+	tokens[last][0] = tokens[last][s - 1] = ' ';
+
+	// Also construct an alternate set of strings like " token" to
+	// be searched for only at the end of each name field.
+	end_tokens.push_back(tokens[last].substr(0, s - 1));
+      }
+      else {
+	tokens.push_back(temp_tokens[i]);
+	end_tokens.push_back(temp_tokens[i]);
+      }
+    }
+
+    if (start_tokens.size() > 1) // invalid syntax: more than one "^token"
+      return;
   }
-  StringList tokens(searchstring);
   
   // loop over data files
   iterate (StringList, filelist, filename_ptr) {
@@ -288,14 +353,35 @@ void StarArray::Search(const std::string &search, StringList filelist,
 	if (exactmatch)
 	  result = (searchstring == *name_ptr);
 	else {
+	  // Search on substrings; all substrings must match within the
+	  // same star name.  Replace matching substrings with *'s as we go
+	  // so that found substrings don't overlap (e.g. if someone searches
+	  // for "Tau Tauri").  We don't delete them completely since that
+	  // could screw up word matching.
 	  result = true;
 	  string temp = *name_ptr;
-	  iterate (StringList, tokens, token_ptr) {
-	    // search on substrings; all substrings must match within the
-	    // same star name.  Erase matching substrings as we go so that
-	    // found substrings don't overlap (e.g. if someone searches
-	    // for "Tau Tauri").
-	    bool found = starstrings::find_and_replace(temp, *token_ptr, "");
+	  if (start_tokens.size()) { // start_tokens.size() == 0 or 1
+	    unsigned s = start_tokens[0].size();
+	    if (start_tokens[0] == temp.substr(0, s))
+	      starstrings::find_and_replace(temp, start_tokens[0], "*");
+	    else
+	      result = false;
+	  }
+	  if (result)
+	  for (unsigned i = 0; i < tokens.size(); i++) {
+	    bool found = false;
+	    
+	    // If this is a word we can safely replace it with a space;
+	    // otherwise it must be replaced with a '*'
+	    string replacement = ((tokens[i][0] == ' ') ? " " : "*");
+	    if (starstrings::find_and_replace(temp, end_tokens[i],
+				    replacement,
+				    temp.size() - end_tokens[i].size()))
+	      found = true;
+	    else if (starstrings::find_and_replace(temp, tokens[i],
+				    replacement))
+	      found = true;
+
 	    result = (result && found);
 	    if (!result) break;
 	  }
@@ -442,7 +528,7 @@ bool StarArray::SetRules(const Rules &rules, star_changetype_t ruleschange)
 	      (*star_ptr).SetLabel(true);
 	}
       }
-    case NO_CHANGE:
+    case UNITS_CHANGE: case NO_CHANGE:
       // do nothing; these changes are taken care of in StarArray::Display()
       break;
 
@@ -584,23 +670,34 @@ void StarArray::drawlegend(StarViewer *sv) const
   sra += starstrings::ra_to_str(gridposn.getPhi(), ' ',
 				ArrayRules.CelestialCoords, true);
   sdec += starstrings::dec_to_str(gridposn.getTheta(), ' ', true);
-  sdist = starstrings::ftoa(ArrayRules.ChartLocation.magnitude(), 5) + " " +
-	  /* TRANSLATORS: "LY" is the abbreviation for "light-years". */
-	  _("LY from Earth");
+  sdist = starstrings::distance_to_str(ArrayRules.ChartLocation.magnitude(),
+				       ArrayRules.ChartUnits)
+	  + " " + _("from Earth");
   srad = string(_("Chart Radius")) + ": " +
-	  starstrings::ftoa(ArrayRules.ChartRadius, 5) + " " +
-	  /* TRANSLATORS: "LY" is the abbreviation for "light-years". */
-	  _("LY");
+	  starstrings::distance_to_str(ArrayRules.ChartRadius,
+				       ArrayRules.ChartUnits);
   sdmag = 
 	  /* TRANSLATORS: "Mag" is the abbreviation for "magnitude",
 	     the astronomical brightness of an object. */
-	  string(_("Dim Mag")) + ": " +
-          starstrings::ftoa(ArrayRules.ChartDimmestMagnitude, 4);
+	  string(_("Dim Mag")) + ": " + starstrings::ftoa(
+	      starmath::roundoff(ArrayRules.ChartDimmestMagnitude, 2), 4);
   sbmag =
 	  /* TRANSLATORS: "Mag" is the abbreviation for "magnitude",
 	     the astronomical brightness of an object. */
-	  string(_("Bright Mag")) + ": " +
-	  starstrings::ftoa(ArrayRules.ChartBrightestMagnitude, 4);
+	  string(_("Bright Mag")) + ": " + starstrings::ftoa(
+	      starmath::roundoff(ArrayRules.ChartBrightestMagnitude, 2), 4);
+
+  // Calculate dimmest apparent magnitude possible for any star permitted
+  // to appear on the chart, and display it.  Useful information in case
+  // the user knows s/he is using a star catalog with a hard lower limit
+  // based on apparent magnitude.
+  double max_distance = ArrayRules.ChartLocation.magnitude()
+			+ ArrayRules.ChartRadius;
+  float dimmest_apparent_mag = starmath::roundoff(
+    starmath::get_appmag(ArrayRules.ChartDimmestMagnitude, max_distance), 2);
+  sdmag += " (";
+  sdmag += starstrings::ftoa(dimmest_apparent_mag, 4);
+  sdmag += ")";
   
   sv->setcolor(BOLD_COLOR); sv->drawtext(_("CHART STATUS"), 15, ybase += 15);
   sv->setcolor(LABEL_COLOR);
@@ -618,8 +715,10 @@ void StarArray::drawlegend(StarViewer *sv) const
       <= 0.0)
     sv->setcolor(ERROR_COLOR);
   else sv->setcolor(LABEL_COLOR);
-  sv->drawtext(sbmag, 5, ybase += 15);
   sv->drawtext(sdmag, 5, ybase += 15);
+  // Show bright mag only if it's been changed from the default
+  if (ArrayRules.ChartBrightestMagnitude > -25)
+    sv->drawtext(sbmag, 5, ybase += 15);
 
   // then draw the legend (which doubles as a display of which spectral
   //  classes are currently turned on) in the upper right
