@@ -38,7 +38,7 @@
 enum star_changetype_t {
   FILE_CHANGE,        LOCATION_CHANGE, RADIUS_CHANGE,
   COORDINATE_CHANGE,  FILTER_CHANGE,   ORIENTATION_CHANGE,
-  DECORATIONS_CHANGE, NO_CHANGE
+  DECORATIONS_CHANGE, UNITS_CHANGE,    NO_CHANGE
 };
 
 // Specify the character used to separate records in the data files
