@@ -47,6 +47,14 @@ class StringList : public std::vector<std::string> {
   std::string flatten(char spacer = ' ') const;
   char ** c_str() const;
 };
+  
+// Enum of distance units
+enum distance_unit { DIST_LY, DIST_PC, DIST_KPC,
+		     DIST_AU, DIST_KM, DIST_MI, N_DIST };
+
+extern double distance_conversion[N_DIST];
+extern const char * distance_name[N_DIST];
+extern const char * distance_cname[N_DIST];
 
 // Useful string functions
 
@@ -76,14 +84,19 @@ namespace starstrings {
   void utf8ize(std::string &);
   void addgreek(std::string &);
   bool find_and_replace(std::string & haystack, const std::string & needle,
-			const std::string & replacement);
+			const std::string & replacement, unsigned posn = 0);
 
   // compare strings
   bool compare(const std::string &, const std::string &);
   bool case_compare(const std::string &, const std::string &);
   bool compare_n(const std::string &, const std::string &, size_t length);
   bool case_compare_n(const std::string &, const std::string &, size_t length);
-    
+
+  // Optimization: do case-insensitive comparison assuming that first
+  // argument is already converted to uppercase:
+  bool case_compare1(const std::string &, const std::string &);
+  bool case_compare1_n(const std::string &, const std::string &, size_t len);
+
   // Functions to convert between string and radian representations of
   // coordinates.  "symbols" decides whether or not to append h,m,s,deg,',"
   StringList dec_to_strs(double declination, bool symbols = false);  
@@ -104,7 +117,11 @@ namespace starstrings {
   double str_to_ra(const std::string &rastring, char tokenizer = ' ',
 		   bool celestial_coords = true);
 
-}; // end of namespace
+  distance_unit str_to_unit(const std::string &);
+  std::string distance_to_str(double distance_ly, distance_unit unit);
+  std::string distance_to_str(double distance_ly, const distance_unit *arr = 0);
+
+} // end of namespace
 
 
 // inline function definitions
@@ -216,9 +233,12 @@ starstrings::stripspace(std::string &s)
 inline bool
 starstrings::find_and_replace(std::string & haystack,
 			      const std::string & needle,
-			      const std::string & replacement)
+			      const std::string & replacement,
+			      unsigned start_posn)
 {
-  size_t posn = haystack.find(needle);
+  if (start_posn >= haystack.size()) return false;
+
+  size_t posn = haystack.find(needle, start_posn);
   if (posn < haystack.size()) {
     haystack.replace(posn, needle.size(), replacement);
     return true;
@@ -258,6 +278,10 @@ inline bool
 starstrings::case_compare(const std::string &s1, const std::string &s2)
 { return uppercase(s1) == uppercase(s2); }
 
+inline bool
+starstrings::case_compare1(const std::string &s1, const std::string &s2)
+{ return s1 == uppercase(s2); }
+
 inline bool
 starstrings::compare_n(const std::string &s1, const std::string &s2,
 		       size_t n)
@@ -268,6 +292,11 @@ starstrings::case_compare_n(const std::string &s1, const std::string &s2,
 			    size_t n)
 { return compare_n(uppercase(s1), uppercase(s2), n); }
   
+inline bool
+starstrings::case_compare1_n(const std::string &s1, const std::string &s2,
+			    size_t n)
+{ return compare_n(s1, uppercase(s2), n); }
+  
 inline std::string
 starstrings::dec_to_str(double declination, char spacer, bool symbols)
 { return dec_to_strs(declination, symbols).flatten(spacer); }
