@@ -34,9 +34,4 @@
 #define PROGNAME "starconvert"
 #endif
 
-
-const double LY_PER_PC = 3.2616;
-static const char *BADSTARNAME = "***error***";
-static const Star BADSTAR = Star(BADSTARNAME);
-
 #endif // #ifndef _CONVERT_H
