@@ -454,21 +454,26 @@ void parse_config_subst(const StringList &strlist,
 			namedata *nmd, unsigned int lineno)
 {
   static int count = 0;
+  string upper(strlist[0]);
+  starstrings::toupper(upper);
+  
+  // Use a string that is converted to all-caps if we are case-insensitive
+  string strlist0 = nmd->isSubstCaseSensitive ? strlist[0] : upper;
 
   count++;
-  if (count == 1 && starstrings::case_compare(strlist[0], "case-sensitive")) {
+  if (count == 1 && starstrings::compare(upper, "CASE-SENSITIVE")) {
     nmd->isSubstCaseSensitive = true;
     if (strlist.size() > 1)
       printerror(TOOMANYARGS, "case-sensitive", 0, lineno);
   }
-  else if (count == 1 && starstrings::case_compare(strlist[0], "case-insensitive")) {
+  else if (count == 1 && starstrings::compare(upper, "CASE-INSENSITIVE")) {
     if (strlist.size() > 1)
       printerror(TOOMANYARGS, "case-insensitive", 0, lineno);
   }
   else if (count == 1 && strlist.size() == 1)
     printerror(BADKEYWORD, strlist[0], 0, lineno);
   else if (strlist.size() == 2)
-    nmd->substs.push_back(substitution(strlist[0], strlist[1], 0));
+    nmd->substs.push_back(substitution(strlist0, strlist[1], 0));
   else if (strlist.size() == 3) {
     int posn = starmath::atoi(strlist[2]) - 1;
     if (posn < 0)
@@ -476,7 +481,7 @@ void parse_config_subst(const StringList &strlist,
 "Warning!  Third item in a substitution rule must be greater than 0,\n\
 but is `%s' on line %d; ignoring."), strlist[2].c_str(), lineno) << endl;
     else
-      nmd->substs.push_back(substitution(strlist[0], strlist[1], posn));
+      nmd->substs.push_back(substitution(strlist0, strlist[1], posn));
   }
   else
     cerr << starstrings::ssprintf(_(
