@@ -88,7 +88,7 @@ void my_gtk_position_dialog( GtkWidget **entrybox, GtkWidget *insertionbox,
 
   for (int i = 0; i < 6; i++) {
     starstrings::stripspace(boxentries[i]);
-    entrybox[i] = gtk_entry_new_with_max_length (5);
+    entrybox[i] = gtk_entry_new_with_max_length (10);
     gtk_entry_set_text (GTK_ENTRY (entrybox[i]), boxentries[i].c_str());
     gtk_widget_set_usize (entrybox[i], 40, 20);
     gtk_widget_show (entrybox[i]);
@@ -134,7 +134,19 @@ void chart_define_callback(GtkWidget *widget, gpointer chart_define_data)
   GtkWidget **data = (GtkWidget **)chart_define_data;
   double oldchartradius = globals::chart_rules.ChartRadius;
   double newchartradius = starmath::atof(
-		          gtk_entry_get_text (GTK_ENTRY (data[7])));
+		          gtk_entry_get_text (GTK_ENTRY (data[8])))
+		  / distance_conversion[globals::chart_rules.ChartUnits[1]];
+  
+  if (newchartradius <= 0) {
+    my_gtk_error (GTK_WINDOW (definedialog),
+	  starstrings::ssprintf(
+_("You have entered a negative or\n\
+zero value (%s %s) for the chart radius!\n\
+Ignoring the change in this parameter."),
+	    gtk_entry_get_text (GTK_ENTRY (data[8])),
+	    distance_name[globals::chart_rules.ChartUnits[1]]).c_str());
+    newchartradius = oldchartradius;
+  }
 
   globals::chart_rules.ChartLocation =
     SolidAngle( starstrings::strs_to_ra(
@@ -148,7 +160,8 @@ void chart_define_callback(GtkWidget *widget, gpointer chart_define_data)
 			     gtk_entry_get_text (GTK_ENTRY (data[5]))
 			   )
 		).toCartesian(starmath::atof(
-				gtk_entry_get_text(GTK_ENTRY(data[6]))));
+				gtk_entry_get_text(GTK_ENTRY(data[6])))
+		  / distance_conversion[globals::chart_rules.ChartUnits[1]]);
   
   globals::chart_rules.ChartRadius = newchartradius;
 
@@ -200,17 +213,42 @@ void chart_define_defaults(GtkWidget *widget, gpointer dummy /* not used */)
 void chart_define_search(GtkWidget *widget, gpointer chart_define_data)
 {
   GtkWidget **data = (GtkWidget **)chart_define_data;
-  string starname = gtk_entry_get_text (GTK_ENTRY (data[8]));
+  string starname = gtk_entry_get_text (GTK_ENTRY (data[7]));
   StarArray searchresults;
 
   if (starstrings::isempty(starname)) return;
 
+  StringList ra, dec;
+
+  if (starname == string("[") + _("Chart Center") + string("]")) {
+    SolidAngle s = globals::chart_rules.ChartLocation.toSpherical();
+    //if (globals::chart_rules.CelestialCoords == GALACTIC)
+      //s = s.toGalactic();
+
+    ra = starstrings::ra_to_strs(s.getPhi(),
+				 globals::chart_rules.CelestialCoords);
+    dec = starstrings::dec_to_strs(s.getTheta());
+
+    gtk_entry_set_text (GTK_ENTRY (data[6]), starstrings::ftoa(
+	globals::chart_rules.ChartLocation.magnitude()
+	* distance_conversion[globals::chart_rules.ChartUnits[1]], 8).c_str());
+  }
+  else {
+
   // Try several different kinds of searches, order of most to least specific
   searchresults.Search(starname, globals::chart_rules.ChartFileNames,
 		       globals::chart_rules,
 		       /* not case-sensitive */ false,
 		       /* exact match        */ true,
 		       /* exit on match      */ true);
+
+  // If the search string was quoted, an exact match was forced, so skip
+  // the remaining redundant search attempts
+  unsigned size = starname.size();
+  if (size > 2 && starname[0] == '"' && starname[size - 1] == '"')
+    /* nothing */;
+  else {
+
   if (! searchresults.size())
     searchresults.Search(starname, globals::chart_rules.ChartFileNames,
 			 globals::chart_rules,
@@ -223,7 +261,8 @@ void chart_define_search(GtkWidget *widget, gpointer chart_define_data)
 			 /* not case-sensitive */ false,
 			 /* substring match    */ false,
 			 /* exit on match      */ true);
-  
+  } // end "else"
+
   // if we don't find anything, leave a message in the star name entry
   //  box and return
   if (! searchresults.size()) {
@@ -243,56 +282,47 @@ void chart_define_search(GtkWidget *widget, gpointer chart_define_data)
   //  list of star characteristics?
   unsigned int nameplace = starmath::atoi(infolist[0]);
   unsigned int infolistplace = nameplace ? nameplace + 9 : 1;
-  gtk_entry_set_text (GTK_ENTRY (data[8]), infolist[infolistplace].c_str());
+  gtk_entry_set_text (GTK_ENTRY (data[7]), infolist[infolistplace].c_str());
 
-  StringList ra(infolist[2], SUBFIELD_DELIMITER);
-  StringList dec(infolist[3], SUBFIELD_DELIMITER);
+  ra = StringList(infolist[2], SUBFIELD_DELIMITER);
+  dec = StringList(infolist[3], SUBFIELD_DELIMITER);
 
   if (ra[0] == string(_("N/A")))
-    ra = dec = StringList("0,0,0", ',');
+    ra = dec = StringList("00,00,00", ',');
+
+  gtk_entry_set_text (GTK_ENTRY (data[6]), starstrings::ftoa(
+      searchresults[0].GetStarDistance()
+      * distance_conversion[globals::chart_rules.ChartUnits[1]], 8).c_str());
+  } // end "else"
 
   for (unsigned int i = 0; i < 3; i++) {
     gtk_entry_set_text (GTK_ENTRY (data[i]), ra[i].c_str());
     gtk_entry_set_text (GTK_ENTRY (data[i + 3]), dec[i].c_str());
   }
-  gtk_entry_set_text (GTK_ENTRY (data[6]), infolist[4].c_str());
 
   return;
 }
 
-/* This function has lots of GTK gui boilerplate because it is the most 
- *  complicated of the dialog boxes.  Sorry about that. */
-void chart_define()
+/* my_gtk_star_coordinates(): Create a dialog that permits entering a
+ * star name and clicking "Search" to enter coordinates, or entering
+ * coordinates by hand.  Dialog is packed into an existing vbox passed
+ * in as an argument.
+ *
+ * entryboxes must be an array of or pointer to at least 8 GtkWidget *'s
+ * and is assumed to have the following layout:
+ *
+ * entryboxes[0] through [5] - controlled by my_gtk_position_dialog()
+ *                             gives the two angular coordinates
+ * entryboxes[6]             - distance of coordinate from Sun
+ * entryboxes[7]             - entry box for star name
+ *
+ * */
+void my_gtk_star_coordinates(GtkWidget ** entryboxes, GtkWidget * main_vbox)
 {
-  GtkWidget *main_vbox, *query;
-  GtkWidget *lookup_frame, *lookup_hbox, *lookup_entry, *search;
-  GtkWidget *coord_frame, *coord_vbox;
-  GtkWidget *coord_entry[6], *distance_hbox, *distance_label, *distance_entry;
-  GtkWidget *chartrad_hbox, *chartrad_label, *chartrad_entry;
-  GtkWidget *OK, *defaults, *cancel;
-  static GtkWidget *chart_define_data[9];
-  string chartrad_text, distance_text;
-
-  // The window
-  definedialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_policy (GTK_WINDOW (definedialog), false, false, true);
-  gtk_window_set_title (GTK_WINDOW (definedialog), _("StarPlot: Define Chart"));
-  gtk_window_set_modal (GTK_WINDOW (definedialog), true);
-  g_signal_connect (G_OBJECT (definedialog), "destroy",
-		      G_CALLBACK (gtk_widget_destroy), NULL);
-
-  // Pack it vertically
-  main_vbox = gtk_vbox_new (false, 5);
-  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 10);
-  gtk_container_add (GTK_CONTAINER (definedialog), main_vbox);
-  gtk_widget_show (main_vbox);
-
-  // Query text
-  query = gtk_label_new (
-      _("Where should the chart origin be located?"));
-  gtk_box_pack_start (GTK_BOX (main_vbox), query, false, false, 0);
-  gtk_misc_set_alignment (GTK_MISC (query), 0.0F, 0.0F);
-  gtk_widget_show (query);
+  GtkWidget * lookup_frame, * lookup_hbox, * search;
+  GtkWidget * coord_frame, * coord_vbox;
+  GtkWidget * distance_hbox, * distance_label;
+  string distance_text;
 
   // First element: a frame containing an hbox containing the star search
   //  entry box
@@ -306,9 +336,9 @@ void chart_define()
   gtk_container_set_border_width (GTK_CONTAINER (lookup_hbox), 5);
   gtk_widget_show (lookup_hbox);
   
-  lookup_entry = gtk_entry_new();
-  gtk_box_pack_start (GTK_BOX (lookup_hbox), lookup_entry, true, true, 5);
-  gtk_widget_show (lookup_entry);
+  entryboxes[7] = gtk_entry_new();
+  gtk_box_pack_start (GTK_BOX (lookup_hbox), entryboxes[7], true, true, 5);
+  gtk_widget_show (entryboxes[7]);
 
   search = gtk_button_new_with_mnemonic (_("_Search"));
   gtk_box_pack_start (GTK_BOX (lookup_hbox), search, false, false, 5);
@@ -328,7 +358,8 @@ void chart_define()
   gtk_container_set_border_width (GTK_CONTAINER (coord_vbox), 5);
   gtk_widget_show (coord_vbox);
 
-  my_gtk_position_dialog (coord_entry, coord_vbox,
+  // fills in entryboxes[0] through [5]
+  my_gtk_position_dialog (entryboxes, coord_vbox,
 			  globals::chart_rules.ChartLocation.toSpherical(),
 			  globals::chart_rules.CelestialCoords);
 
@@ -336,56 +367,90 @@ void chart_define()
   gtk_box_pack_start (GTK_BOX (coord_vbox), distance_hbox, false, false, 5);
   gtk_widget_show (distance_hbox);
 
-  distance_label = gtk_label_new (
-      /* TRANSLATORS: "LY" is the abbreviation for "light-years". */
-      _("Distance of chart origin from Sun (LY):"));
+  distance_label = gtk_label_new ((string(
+      _("Distance of position from Sun")) + " ("
+      + distance_name[globals::chart_rules.ChartUnits[1]] + "):").c_str());
   gtk_box_pack_start (GTK_BOX (distance_hbox), distance_label, false, false, 5);
   gtk_widget_show (distance_label);
 
   distance_text =
-    starstrings::ftoa(globals::chart_rules.ChartLocation.magnitude());
-  distance_entry = gtk_entry_new_with_max_length (12);
-  gtk_box_pack_start (GTK_BOX(distance_hbox), distance_entry, false, false, 5);
-  gtk_entry_set_text (GTK_ENTRY (distance_entry), distance_text.c_str());
-  gtk_widget_set_usize (distance_entry, 80, 20);
-  gtk_widget_show (distance_entry);
+    starstrings::ftoa(globals::chart_rules.ChartLocation.magnitude()
+	* distance_conversion[globals::chart_rules.ChartUnits[1]], 8);
+  entryboxes[6] = gtk_entry_new_with_max_length (14);
+  gtk_box_pack_start (GTK_BOX(distance_hbox), entryboxes[6], false, false, 5);
+  gtk_entry_set_text (GTK_ENTRY (entryboxes[6]), distance_text.c_str());
+  gtk_widget_set_usize (entryboxes[6], 80, 20);
+  gtk_widget_show (entryboxes[6]);
+
+  // Connect the buttons to signals
+  g_signal_connect (G_OBJECT (search), "clicked",
+		      G_CALLBACK (chart_define_search),
+		      entryboxes);
+  g_signal_connect (G_OBJECT (entryboxes[7]), "activate",
+		      G_CALLBACK (chart_define_search),
+		      entryboxes);
+}
+
+/* This function has lots of GTK gui boilerplate because it is the most 
+ *  complicated of the dialog boxes.  Sorry about that. */
+void chart_define()
+{
+  GtkWidget *main_vbox, *query;
+  GtkWidget *chartrad_hbox, *chartrad_label, *chartrad_entry;
+  GtkWidget *OK, *defaults, *cancel;
+  static GtkWidget *chart_define_data[9];
+  string chartrad_text;
+
+  // The window
+  definedialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_resizable (GTK_WINDOW (definedialog), false);
+  gtk_window_set_title (GTK_WINDOW (definedialog), _("StarPlot: Define Chart"));
+  gtk_window_set_modal (GTK_WINDOW (definedialog), true);
+  g_signal_connect (G_OBJECT (definedialog), "destroy",
+		      G_CALLBACK (gtk_widget_destroy), NULL);
+
+  // Pack it vertically
+  main_vbox = gtk_vbox_new (false, 5);
+  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 10);
+  gtk_container_add (GTK_CONTAINER (definedialog), main_vbox);
+  gtk_widget_show (main_vbox);
+
+  // Query text
+  query = gtk_label_new (
+      _("Where should the chart origin be located?"));
+  gtk_box_pack_start (GTK_BOX (main_vbox), query, false, false, 0);
+  gtk_misc_set_alignment (GTK_MISC (query), 0.0F, 0.0F);
+  gtk_widget_show (query);
+
+  // Drop in the two elements of the star search frame
+  // (fills in chart_define_data[0] through [7])
+  my_gtk_star_coordinates(chart_define_data, main_vbox);
 
   // Third element: an hbox containing the Chart Radius entry box
   chartrad_hbox = gtk_hbox_new (false, 5);
   gtk_box_pack_start (GTK_BOX (main_vbox), chartrad_hbox, false, false, 10);
   gtk_widget_show (chartrad_hbox);
 
-  chartrad_label = gtk_label_new (
-      /* TRANSLATORS: "LY" is the abbreviation for "light-years". */
-      _("Radius of chart (LY):"));
+  chartrad_label = gtk_label_new ((string(
+      _("Radius of chart")) + " ("
+      + distance_name[globals::chart_rules.ChartUnits[1]] + "):").c_str());
   gtk_box_pack_start (GTK_BOX(chartrad_hbox), chartrad_label, false, false, 5);
   gtk_misc_set_alignment (GTK_MISC (chartrad_label), 0.0F, 0.0F);
   gtk_widget_show (chartrad_label);
 
-  chartrad_text = starstrings::ftoa(globals::chart_rules.ChartRadius);
-  chartrad_entry = gtk_entry_new_with_max_length (12);
+  chartrad_text = starstrings::ftoa(globals::chart_rules.ChartRadius
+      * distance_conversion[globals::chart_rules.ChartUnits[1]], 8);
+  chartrad_entry = gtk_entry_new_with_max_length (14);
   gtk_box_pack_start(GTK_BOX(chartrad_hbox), chartrad_entry, false, false, 10);
   gtk_entry_set_text (GTK_ENTRY (chartrad_entry), chartrad_text.c_str());
   gtk_widget_set_usize (chartrad_entry, 80, 20);
   gtk_widget_show (chartrad_entry);
+  chart_define_data[8] = chartrad_entry;
 
   // Last element: the OK/Defaults/Cancel buttons
   my_gtk_button_bar(&OK, &defaults, &cancel, main_vbox);
 
-  // Set up chart_define_data
-  for (int i = 0; i < 6; i++)
-    chart_define_data[i] = coord_entry[i];
-  chart_define_data[6] = distance_entry;
-  chart_define_data[7] = chartrad_entry;
-  chart_define_data[8] = lookup_entry;
-
   // Connect the buttons to signals
-  g_signal_connect (G_OBJECT (search), "clicked",
-		      G_CALLBACK (chart_define_search),
-		      chart_define_data);
-  g_signal_connect (G_OBJECT (lookup_entry), "activate",
-		      G_CALLBACK (chart_define_search),
-		      chart_define_data);
   g_signal_connect (G_OBJECT (OK), "clicked", 
 		      G_CALLBACK (chart_define_callback),
 		      chart_define_data);
@@ -445,7 +510,7 @@ void chart_orient()
 
   // The window
   orientdialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_policy (GTK_WINDOW (orientdialog), false, false, true);
+  gtk_window_set_resizable (GTK_WINDOW (orientdialog), false);
   gtk_window_set_title (GTK_WINDOW (orientdialog), _("StarPlot: Orientation"));
   gtk_window_set_modal (GTK_WINDOW (orientdialog), true);
   g_signal_connect (G_OBJECT (orientdialog), "destroy",
@@ -555,7 +620,7 @@ void chart_filter()
 
   // The window
   filterdialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_policy (GTK_WINDOW (filterdialog), false, false, true);
+  gtk_window_set_resizable (GTK_WINDOW (filterdialog), false);
   gtk_window_set_title (GTK_WINDOW (filterdialog), _("StarPlot: Star Filter"));
   gtk_window_set_modal (GTK_WINDOW (filterdialog), true);
   g_signal_connect (G_OBJECT (filterdialog), "destroy",
