@@ -24,14 +24,16 @@
 
 #include <gtk/gtk.h>
 #include "starplot.h"
+#include <cstring>
+#include <cstdlib> // for getenv()
 
 using std::endl;
 using std::string;
 
-bool read_params(const string & filename);
-bool write_params(const string & filename, const string & header);
-void my_gtk_file_select (void (*)(GtkWidget *, GtkFileSelection *),
-			 const string &);
+static bool read_params(const string & filename);
+static bool write_params(const string & filename, const string & header);
+static void my_gtk_file_select (void (*)(GtkWidget *, GtkFileSelection *),
+				const string &, const string &);
 
 void starplot_quit()
 {
@@ -48,8 +50,7 @@ void starplot_quit()
  *  the selection of the help browser program. */
 
 /* This opens a new file */
-void file_ok_sel( GtkWidget        *w,
-		  GtkFileSelection *fs )
+static void file_ok_sel(GtkWidget *w, GtkFileSelection *fs)
 {
   globals::chart_rules.ChartFileNames = StringList(1,
 		  gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)));
@@ -57,8 +58,7 @@ void file_ok_sel( GtkWidget        *w,
 }
 
 /* This merges the selected file with the other(s) already open */
-void merge_ok_sel( GtkWidget        *w,
-		   GtkFileSelection *fs )
+static void merge_ok_sel(GtkWidget *w, GtkFileSelection *fs)
 {
   string filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
   				
@@ -73,8 +73,7 @@ void merge_ok_sel( GtkWidget        *w,
 }
 
 /* This opens a new set of chart parameters */
-void param_open_sel( GtkWidget        *w,
-		     GtkFileSelection *fs )
+static void param_open_sel(GtkWidget *w, GtkFileSelection *fs)
 {
   string filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
   if (read_params(filename))
@@ -82,8 +81,7 @@ void param_open_sel( GtkWidget        *w,
 }
 
 /* This saves the current chart parameters to a file */
-void param_save_sel( GtkWidget        *w,
-		     GtkFileSelection *fs )
+static void param_save_sel(GtkWidget *w, GtkFileSelection *fs)
 {
   string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
   
@@ -93,11 +91,14 @@ void param_save_sel( GtkWidget        *w,
 }
 
 /* This saves the current chart as a PNG image file */
-void file_save_sel( GtkWidget *w, GtkFileSelection *fs )
+static void file_save_sel(GtkWidget *w, GtkFileSelection *fs)
 {
-  char * filename = new char[std::strlen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs))) + 1];
-  strcpy(filename, gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
-  
+  string filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
+  unsigned len = filename.size();
+
+  if (len <= 4 || filename.substr(len - 4) != ".png")
+    filename += ".png";
+
   GError *error = NULL;
   GdkPixbuf *buffer =
     gdk_pixbuf_get_from_drawable (NULL, globals::program_pixmap,
@@ -107,74 +108,98 @@ void file_save_sel( GtkWidget *w, GtkFileSelection *fs )
     my_gtk_error (0,
 	_("Sorry, an error occurred\ncreating the buffer."));
   
-  else if (!gdk_pixbuf_save (buffer, filename, "png", &error, NULL))
+  else if (!gdk_pixbuf_save (buffer, filename.c_str(), "png", &error, NULL))
     my_gtk_error (0, starstrings::ssprintf(
-	  _("Sorry, couldn't create file \"%s\"."), filename).c_str());
-
-  delete [] filename;
+	  _("Sorry, couldn't create file \"%s\"."), filename.c_str()).c_str());
 }
 
+#ifndef BROWSER
 /* This selects a help file browser */
-void help_ok_sel( GtkWidget        *w,
-		  GtkFileSelection *fs )
+static void help_ok_sel(GtkWidget *w, GtkFileSelection *fs)
 {
   globals::program_help_browser =
     gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
 }
 
 /* This selects a help file browser and also opens the help documentation */
-void help_open_sel( GtkWidget        *w,
-		    GtkFileSelection *fs )
+static void help_open_sel(GtkWidget *w, GtkFileSelection *fs)
 {
   help_ok_sel(w, fs);
   helpdocs();
 }
-
+#endif
 
 /* We need several functions for the menu generator, plus one for the case
  *  where a help browser must be selected before opening help docs,
  *  but they all call the same interface, just choose the correct file
  *  selection function */
+static std::string get_local_dir()
+{
+  using namespace std; // is getenv() in std or :: ?
+  string str = "~";
+  
+  const char * result = getenv("PWD");
+  if (result && strlen(result))   str = result;
+  if (str[str.size() - 1] != '/') str += '/';
+  return str;
+}
+
 void file_open()
 {
   my_gtk_file_select(file_ok_sel,
-      _("StarPlot: Select data file to open"));
+      _("StarPlot: Select data file to open"), DATADIR "/");
 }
 void file_merge()  
 {
   my_gtk_file_select(merge_ok_sel,
-      _("StarPlot: Select data file to merge"));
+      _("StarPlot: Select data file to merge"), DATADIR "/");
 }
 void params_open()
 {
   my_gtk_file_select(param_open_sel,
-      _("StarPlot: Select parameter file to use"));
+      _("StarPlot: Select parameter file to use"), get_local_dir().c_str());
 }
 void params_save()
 {
   my_gtk_file_select(param_save_sel,
-      _("StarPlot: Save chart parameters as:"));
+      _("StarPlot: Save chart parameters as:"), get_local_dir().c_str());
 }
 void file_save()
 {
   my_gtk_file_select(file_save_sel,
-      _("StarPlot: Save chart as PNG image"));
-}
-void help_select() 
-{
-  my_gtk_file_select(help_ok_sel,
-      _("StarPlot: Select help browser program"));
-}
-void help_select_and_open()
-{
-  my_gtk_file_select(help_open_sel,
-      _("StarPlot: Select help browser program"));
+      _("StarPlot: Save chart as PNG image"), get_local_dir().c_str());
 }
 
+#ifndef BROWSER
+void help_select() 
+{
+  string file;
+  if (starstrings::isempty(globals::program_help_browser))
+    file = "/usr/bin/";
+  else
+    file = globals::program_help_browser;
+  my_gtk_file_select(help_ok_sel,
+      _("StarPlot: Select help browser program"), file);
+}
+#endif
+
+void help_select_and_open()
+{
+#ifndef BROWSER
+  string file;
+  if (starstrings::isempty(globals::program_help_browser))
+    file = "/usr/bin/";
+  else
+    file = globals::program_help_browser;
+  my_gtk_file_select(help_open_sel,
+      _("StarPlot: Select help browser program"), file);
+#endif
+}
 
 /* The interface which creates the dialog */
 void my_gtk_file_select 
-(void (*select_function)(GtkWidget *, GtkFileSelection *), const string &title)
+(void (*select_function)(GtkWidget *, GtkFileSelection *), const string &title,
+ const string &startfile)
 { 
   GtkWidget *filew;
 
@@ -199,20 +224,7 @@ void my_gtk_file_select
 			     G_OBJECT (filew));
 
   /* Set the default filename */
-  if (select_function == help_ok_sel || select_function == help_open_sel) {
-    if (starstrings::isempty(globals::program_help_browser))
-      gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew),
-				       "/usr/bin/");
-    else
-      gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew),
-				       globals::program_help_browser.c_str());
-  }
-  else if (select_function == param_open_sel ||
-	   select_function == param_save_sel ||
-	   select_function == file_save_sel)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), "~/");
-  else
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), DATADIR "/");
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), startfile.c_str());
 
   gtk_widget_show(filew);
 }
@@ -298,6 +310,22 @@ bool read_params(const string &filename)
 	globals::chart_rules.StarClasses[i] = static_cast<bool>
 	  (starmath::atoi(classes[i]));
     }
+    else if (assign[0] == string("ChartUnits")) {
+      distance_unit units[4];
+      StringList unit_strs(assign[1], ',');
+      unit_strs.stripspace();
+      unsigned int i;
+      for (i = 0; i < unit_strs.size(); i++) {
+	if (i == 4) break;
+	units[i] = starstrings::str_to_unit(unit_strs[i]);
+      }
+      // fill any unfilled entries caused by truncated list in rc file
+      if (!i) { units[0] = DIST_LY; i = 1; }
+      if (i < 4)
+	for (unsigned int j = i; j < 4; j++)
+	  units[j] = units[i - 1];
+      set_distance_units(units, false);
+    }
     else if (assign[0] == string("CelestialCoords"))
       set_coord_item(static_cast<bool>(starmath::atoi(assign[1])), false);
     else if (assign[0] == string("StarDiameters"))
@@ -343,6 +371,8 @@ bool write_params(const string & filename, const string & header)
   iterate (StringList, globals::chart_rules.ChartFileNames, name_ptr)
     paramstream << "DataFile = " << *name_ptr << endl << endl;
 
+  std::streamsize oldprec = paramstream.precision();
+  paramstream.precision(12);
   paramstream << "# " << _("(X, Y, Z) of chart center, in light-years") << endl
 	      << "Location = ( "
 	      << globals::chart_rules.ChartLocation.getX() << ", "
@@ -360,6 +390,7 @@ bool write_params(const string & filename, const string & header)
 	      << globals::chart_rules.ChartOrientation.getPhi()
 	      << ", " << globals::chart_rules.ChartOrientation.getTheta()
 	      << " )" << endl << endl;
+  paramstream.precision(oldprec);
 	      
   paramstream << "MagLimits = [ "
 	      << globals::chart_rules.ChartBrightestMagnitude << ", "
@@ -378,6 +409,7 @@ bool write_params(const string & filename, const string & header)
 	      << _("dwarf") << ", " <<  _("unclassified")
 	      << ", " <<  _("non-stellar") << " }"
 	      << endl;
+  paramstream << "# " << _("1 if displayed, 0 if not displayed") << endl;
   paramstream << "SpecClassDisplay = { ";
   for (unsigned int i = 0; i < 9; i ++)
     paramstream << globals::chart_rules.StarClasses[i] << ", ";
@@ -392,11 +424,24 @@ bool write_params(const string & filename, const string & header)
   paramstream << "StarBars = " << globals::chart_rules.StarBars << endl;
   paramstream << "ChartGrid = " << globals::chart_rules.ChartGrid << endl;
   paramstream << "ChartLegend = " << globals::chart_rules.ChartLegend << endl;
-#ifndef BROWSER
+
+  paramstream << endl << "# ChartUnits: { "
+	      << _("long-range") << ", " <<  _("standard") << ", "
+	      << _("short-range") << ", " << _("very short-range") << " }"
+	      << endl;
+  paramstream << "# " << _("Allowed units:") << " ";
+  for (unsigned int i = 0; i < N_DIST - 1; i++)
+    paramstream << distance_cname[i] << ", ";
+  paramstream << distance_cname[N_DIST - 1] << endl;
+  paramstream << "ChartUnits = { ";
+  for (unsigned int i = 0; i < 3; i++)
+    paramstream << distance_cname[globals::chart_rules.ChartUnits[i]] << ", ";
+  paramstream << distance_cname[globals::chart_rules.ChartUnits[3]] << " }"
+	      << endl << endl;;
+
+  paramstream << "FontDescription = " << globals::display_fontname << endl;
   if (! starstrings::isempty(globals::program_help_browser))
     paramstream << "HelpBrowser = " << globals::program_help_browser << endl;
-#endif
-  paramstream << "FontDescription = " << globals::display_fontname << endl;
 
   paramstream << endl;
   return true;
