@@ -31,7 +31,7 @@
 namespace globals {
   static GtkWidget  *program_hr_window = NULL;
   static GdkPixmap  *program_hr_pixmap = NULL;
-};
+}
 
 
 void hr_redraw_display(GtkWidget *widget)
@@ -123,7 +123,7 @@ void hr_settings()
   
   // The window
   hr_set_dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_policy (GTK_WINDOW (hr_set_dialog), false, false, true);
+  gtk_window_set_resizable (GTK_WINDOW (hr_set_dialog), false);
   gtk_window_set_title (GTK_WINDOW (hr_set_dialog), 
       /* TRANSLATORS: "HR" stands for "Hertzsprung-Russell", a proper name. */
       _("StarPlot: HR Diagram Scale"));
