@@ -24,6 +24,7 @@
 
 #include <gtk/gtk.h>
 #include "starplot.h"
+#include <cstring>
 using std::string;
 
 // ---------------------------------------------------------------------------
@@ -61,13 +62,13 @@ void my_gtk_star_popup(Star &s)
     RA = string(_("Long.")) + ": " + starinfo[2];
     Dec = string(_("Lat.")) + ": " + starinfo[3];
   }
-  Distance = string(_("Distance")) + ": " +
-    starstrings::ftoa(s.GetStarDistance()) + " " + _("LY");
+  Distance = string(_("Distance")) + ": " + starstrings::distance_to_str(
+      s.GetStarDistance(), globals::chart_rules.ChartUnits[1]);
   Class = string(_("Spectral Class")) + ": " + starinfo[5];
   Mag = string(_("Magnitude")) + ": " + starstrings::ftoa(s.GetStarMagnitude());
   
   popup = gtk_dialog_new ();
-  gtk_window_set_policy (GTK_WINDOW (popup), false, false, true);
+  gtk_window_set_resizable (GTK_WINDOW (popup), false);
   gtk_window_set_title (GTK_WINDOW (popup), _("StarPlot: Star Info"));
   gtk_window_set_position (GTK_WINDOW (popup), GTK_WIN_POS_MOUSE);
   
@@ -240,6 +241,8 @@ void my_gtk_list_setup (GtkWidget *box, GtkTreeView **treeviewptr,
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+				       GTK_SHADOW_IN);
   gtk_box_pack_start (GTK_BOX (box), scrolled_window, true, true, 0);
   gtk_widget_show (scrolled_window);
 
@@ -261,7 +264,6 @@ void my_gtk_list_setup (GtkWidget *box, GtkTreeView **treeviewptr,
   gtk_tree_view_set_rules_hint (treeview, true);
   gtk_widget_set_usize (GTK_WIDGET (treeview), 600, 300);
   gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (treeview));
-  //gtk_clist_set_shadow_type (GTK_CLIST (clist), GTK_SHADOW_OUT);
 
   // render the data viewer columns
   GtkTreeViewColumn *column;
@@ -289,7 +291,9 @@ void my_gtk_list_setup (GtkWidget *box, GtkTreeView **treeviewptr,
   }
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (store),
       COL_DEC, signed_coord_sort_func, NULL, NULL);
-  COL_SETUP   ((string(_("Distance")) + " (" + _("LY") + ")").c_str(),
+  COL_SETUP   ((string(_("Distance")) + " ("
+		+ distance_name[globals::chart_rules.ChartUnits[1]]
+		+ ")").c_str(),
 	       COL_DISTANCE);
   gtk_tree_view_column_set_cell_data_func (column, renderer,
       print_double_to_text, (void *)COL_DISTANCE, 0);
@@ -311,7 +315,7 @@ void my_gtk_list_setup (GtkWidget *box, GtkTreeView **treeviewptr,
 namespace globals {
   GtkTreeView *program_treeview = NULL;
   static GtkWidget *program_info_window = NULL;
-};
+}
 
 void info_quit()
 {
@@ -350,6 +354,9 @@ void update_info(GtkTreeView *treeview)
   gtk_tree_view_column_set_title (column,
       globals::chart_rules.CelestialCoords ?
       _("Declination") : _("Gal. Latitude"));
+  column = gtk_tree_view_get_column (treeview, namecol + 3);
+  gtk_tree_view_column_set_title (column, (string(_("Distance")) + " ("
+	+ distance_name[globals::chart_rules.ChartUnits[1]] + ")").c_str());
 
   // update the data
   GtkListStore *store = (GtkListStore *)gtk_tree_view_get_model (treeview);
@@ -365,7 +372,8 @@ void update_info(GtkTreeView *treeview)
 			COL_NAME,  infolist[1].c_str(),
 			COL_RA,    infolist[2].c_str(),
 			COL_DEC,   infolist[3].c_str(),
-			COL_DISTANCE, (*star_ptr).GetStarDistance(),
+			COL_DISTANCE, (*star_ptr).GetStarDistance()
+		    * distance_conversion[globals::chart_rules.ChartUnits[1]],
 			COL_SPCLASS, infolist[5].c_str(),
 			COL_MAGNITUDE, (*star_ptr).GetStarMagnitude(),
 			-1 /* to mark end of args */);
@@ -483,7 +491,8 @@ void search_callback(GtkWidget *widget, gpointer search_data)
 			COL_ALTNAME,  information[1].c_str(),
 			COL_RA,	      information[2].c_str(),
 			COL_DEC,      information[3].c_str(),
-			COL_DISTANCE, (*result_ptr).GetStarDistance(),
+			COL_DISTANCE, (*result_ptr).GetStarDistance()
+		    * distance_conversion[globals::chart_rules.ChartUnits[1]],
 			COL_SPCLASS,  information[5].c_str(),
 			COL_MAGNITUDE,(*result_ptr).GetStarMagnitude(),
 			-1 /* to mark end of args */);
@@ -568,3 +577,138 @@ void starsearch()
   gtk_widget_show_all (searchwindow);
   return;
 }
+
+// ---------------------------------------------------------------------------
+/* This function displays a dialog allowing to calculate the distance
+ * between two stars (or arbitrary coordinates): */
+
+/* Callback function that actually calculates the distance: */
+static void calculate_star_distance(GtkWidget *widget, gpointer star_data)
+{
+  GtkWidget ** coords = (GtkWidget **)star_data;
+
+  int mult[2] = { 1, -1 };
+  Vector3 result;
+
+  for (unsigned i = 0; i < 2; i++) {
+    Vector3 v = SolidAngle( starstrings::strs_to_ra(
+			      gtk_entry_get_text (GTK_ENTRY (coords[8*i + 0])),
+			      gtk_entry_get_text (GTK_ENTRY (coords[8*i + 1])),
+			      gtk_entry_get_text (GTK_ENTRY (coords[8*i + 2])),
+			      globals::chart_rules.CelestialCoords),
+			    starstrings::strs_to_dec(
+			      gtk_entry_get_text (GTK_ENTRY (coords[8*i + 3])),
+			      gtk_entry_get_text (GTK_ENTRY (coords[8*i + 4])),
+			      gtk_entry_get_text (GTK_ENTRY (coords[8*i + 5]))
+			    )
+			  ).toCartesian(starmath::atof(
+			      gtk_entry_get_text(GTK_ENTRY (coords[8*i + 6])))
+		    / distance_conversion[globals::chart_rules.ChartUnits[1]]);
+    result += mult[i] * v;
+  }
+
+  string dist = starstrings::distance_to_str(result.magnitude(),
+					     globals::chart_rules.ChartUnits);
+  gtk_entry_set_text (GTK_ENTRY (coords[2*8]), dist.c_str());
+}
+
+void stardistance(void)
+{
+  GtkWidget *window, *main_hbox, *main_vbox, *title;
+  GtkWidget *star_frame[2], *star_vbox[2];
+  GtkWidget *OK, *calculate, *results_hbox, *results_label;
+  static GtkWidget * star_coords[2*8 + 1];
+
+  // The window
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_resizable (GTK_WINDOW (window), false);
+  gtk_window_set_title (GTK_WINDOW (window), _("StarPlot: Calculate Distance"));
+  gtk_window_set_modal (GTK_WINDOW (window), true);
+  g_signal_connect (G_OBJECT (window), "destroy",
+                      G_CALLBACK (gtk_widget_destroy), NULL);
+
+  // Pack it vertically
+  main_vbox = gtk_vbox_new (false, 5);
+  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 10);
+  gtk_container_add (GTK_CONTAINER (window), main_vbox);
+  gtk_widget_show (main_vbox);
+
+  // Add explanatory title
+  title = gtk_label_new (_(
+"Define the two positions between which to calculate the distance.\n\
+(The default for the first position is the center of the current chart.)"));
+  gtk_box_pack_start (GTK_BOX (main_vbox), title, false, false, 0);
+  gtk_misc_set_alignment (GTK_MISC (title), 0.5F, 0.0F);
+  gtk_widget_show (title);
+
+  // Set up star frames horizontally
+  main_hbox = gtk_hbox_new (true, 20);
+  gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 10);
+  gtk_box_pack_start (GTK_BOX (main_vbox), main_hbox, false, false, 0);
+  gtk_widget_show (main_hbox);
+
+  // Set up star frames
+  for (unsigned i = 0; i < 2; i++) {
+    string s = starstrings::ssprintf(_("Coordinates of position %d:"), i+1);
+    star_frame[i] = gtk_frame_new (s.c_str());
+    gtk_box_pack_start (GTK_BOX (main_hbox), star_frame[i], false, false, 0);
+    gtk_frame_set_label_align (GTK_FRAME (star_frame[i]), 0.5F, 0.0F);
+    gtk_frame_set_shadow_type (GTK_FRAME (star_frame[i]), GTK_SHADOW_NONE);
+    gtk_widget_show (star_frame[i]);
+
+    star_vbox[i] = gtk_vbox_new (false, 5);
+    gtk_container_add (GTK_CONTAINER (star_frame[i]), star_vbox[i]);
+    gtk_container_set_border_width (GTK_CONTAINER (star_vbox[i]), 5);
+    gtk_widget_show (star_vbox[i]);
+
+    my_gtk_star_coordinates(&star_coords[8*i], star_vbox[i]);
+  }
+
+  // Set the default for the second position to be the origin.
+  for (unsigned i = 0; i < 6; i++)
+    gtk_entry_set_text (GTK_ENTRY (star_coords[8 + i]), "00");
+  gtk_entry_set_text (GTK_ENTRY (star_coords[8+6]), "0");
+
+  gtk_entry_set_text (GTK_ENTRY (star_coords[0+7]), 
+      (string("[") + _("Chart Center") + string("]")).c_str());
+  gtk_entry_set_text (GTK_ENTRY (star_coords[8+7]), "Sun");
+
+  // Add calculate button and results field
+  results_hbox = gtk_hbox_new (false, 20);
+  gtk_container_set_border_width (GTK_CONTAINER (results_hbox), 10);
+  gtk_box_pack_start (GTK_BOX (main_vbox), results_hbox, false, false, 0);
+  gtk_widget_show (results_hbox);
+
+  GtkWidget * spacer = gtk_label_new ("");
+  gtk_box_pack_start (GTK_BOX (results_hbox), spacer, true, false, 0);
+  gtk_widget_show (spacer);
+
+  calculate = gtk_button_new_with_mnemonic (_("_Calculate distance"));
+  gtk_box_pack_start (GTK_BOX (results_hbox), calculate, false, false, 10);
+  gtk_widget_show (calculate);
+
+  results_label = gtk_label_new (_("Distance between positions is:"));
+  gtk_box_pack_start (GTK_BOX (results_hbox), results_label, false, false, 0);
+  gtk_misc_set_alignment (GTK_MISC (results_label), 1.0F, 0.5F);
+  gtk_widget_show (results_label);
+
+  star_coords[2*8] = gtk_entry_new();
+  gtk_box_pack_start (GTK_BOX (results_hbox), star_coords[2*8], false,false, 0);
+  gtk_entry_set_editable (GTK_ENTRY (star_coords[2*8]), false);
+  gtk_widget_show (star_coords[2*8]);
+
+  spacer = gtk_label_new ("");
+  gtk_box_pack_start (GTK_BOX (results_hbox), spacer, true, false, 0);
+  gtk_widget_show (spacer);
+  
+  g_signal_connect (G_OBJECT (calculate), "clicked",
+                      G_CALLBACK (calculate_star_distance), star_coords);
+
+  my_gtk_popup_button(&OK, main_vbox, window);
+  
+  // Finally, set the window policy and show it
+  gtk_window_set_focus (GTK_WINDOW (window), OK);
+  gtk_widget_show (window);
+
+  return;
+}
