@@ -50,7 +50,7 @@
 { _("/File/Open _Parameter File..."),"", params_open,  0, STO,   "gtk-open" }, \
 { _("/File/Save _Chart Parameters..."),"",params_save, 0, STO,   "gtk-save" }, \
 { _("/File/-----"),              0,     0,             0, "<Separator>" }, \
-{ _("/File/_Save Chart as PNG..."),   0, file_save,    0, STO,	"gtk-save" }, \
+{ _("/File/_Save Chart as PNG..."),   "", file_save,    0, STO,	"gtk-save" }, \
 { _("/File/-----"),              0,     0,             0, "<Separator>" },     \
 { _("/File/_Quit"),              "",    starplot_quit, 0, STO,   "gtk-quit" }
 
@@ -160,15 +160,42 @@ extern void cel_coords();
 extern void gal_coords();
 
 
+#define OPTIONS_UNIT_MENU_SIZE 6
+extern GtkWidget *options_unit_widgets[OPTIONS_UNIT_MENU_SIZE];
+#define OPTIONS_UNIT_MENU \
+{ _("/Options/Distance _Units"),  0,      0,          0, "<Branch>" },   \
+{ _("/Options/Distance Units/_Light-years, AU, km"), 0, ly_km_units, 0, \
+							      "<RadioItem>" },\
+{ _("/Options/Distance Units/Light-years, AU, _miles"), 0, ly_mi_units, 0, \
+			_("/Options/Distance Units/Light-years, AU, km") }, \
+{ _("/Options/Distance Units/Light-years _only"), 0, ly_units, 0, \
+			_("/Options/Distance Units/Light-years, AU, km") }, \
+{ _("/Options/Distance Units/(_k)pc, AU, km"), 0, pc_km_units, 0, \
+			_("/Options/Distance Units/Light-years, AU, km") }, \
+{ _("/Options/Distance Units/(k)p_c, AU, miles"), 0, pc_mi_units, 0, \
+			_("/Options/Distance Units/Light-years, AU, km") }, \
+{ _("/Options/Distance Units/(k)pc o_nly"), 0, pc_units, 0, \
+			_("/Options/Distance Units/Light-years, AU, km") }
+// defined in menuops.cc:
+extern void ly_km_units();
+extern void pc_km_units();
+extern void ly_mi_units();
+extern void pc_mi_units();
+extern void ly_units();
+extern void pc_units();
+
+
 #define STARS_MENU \
 { _("/_Stars"),                  0,          0,          0, "<Branch>" },      \
 { _("/Stars/Chart _Data..."),  "<control>D", starinfo,   0, "" },              \
 { _("/Stars/_HR Diagram..."),  "<control>H", hrdiagram,  0, "" },              \
-{ _("/Stars/_Search..."),      "<control>S", starsearch, 0, STO,  "gtk-find" }
+{ _("/Stars/_Search..."),      "<control>S", starsearch, 0, STO,  "gtk-find" },\
+{ _("/Stars/_Calculate Distance..."), 0,   stardistance, 0, "" }
 
 // defined in infodialogs.cc:
 extern void starinfo();
 extern void starsearch();
+extern void stardistance();
 // defined in hrdiagram.cc:
 extern void hrdiagram();
 
