@@ -29,14 +29,15 @@
 #include <string.h> // for strdup()
 #include <cstdlib>  // for free()
 #include <map>
+#include <cstring>
 
 #include "starplot.h"
 #include "xpmdata.h"
 
 GtkItemFactoryEntry menu_items[] = {
   FILE_MENU, CHART_MENU, OPTIONS_CUSTOM_MENU, OPTIONS_TOGGLE_MENU,
-  OPTIONS_LABEL_MENU, OPTIONS_DIAMETER_MENU, OPTIONS_COORD_MENU, STARS_MENU,
-  HELP_MENU
+  OPTIONS_LABEL_MENU, OPTIONS_DIAMETER_MENU, OPTIONS_COORD_MENU,
+  OPTIONS_UNIT_MENU, STARS_MENU, HELP_MENU
 };
 
 GtkWidget *options_toggle_widgets[OPTIONS_TOGGLE_MENU_SIZE];
@@ -47,6 +48,8 @@ GtkWidget *options_diameter_widgets[OPTIONS_DIAMETER_MENU_SIZE];
 char options_diameter_names[OPTIONS_DIAMETER_MENU_SIZE][100];
 GtkWidget *options_coord_widgets[OPTIONS_COORD_MENU_SIZE];
 char options_coord_names[OPTIONS_COORD_MENU_SIZE][100];
+GtkWidget *options_unit_widgets[OPTIONS_UNIT_MENU_SIZE];
+char options_unit_names[OPTIONS_UNIT_MENU_SIZE][100];
 
 
 /* set_menu_globals(): A boring function to set all the arrays of menu item
@@ -80,6 +83,13 @@ void set_menu_globals (GtkItemFactory *item_factory)
   strcpy(options_coord_names[GALACTIC],
 	 _("/Options/Coordinate System/Galactic"));
 
+  strcpy(options_unit_names[0], _("/Options/Distance Units/Light-years, AU, km"));
+  strcpy(options_unit_names[1], _("/Options/Distance Units/Light-years, AU, miles"));
+  strcpy(options_unit_names[2], _("/Options/Distance Units/Light-years only"));
+  strcpy(options_unit_names[3], _("/Options/Distance Units/(k)pc, AU, km"));
+  strcpy(options_unit_names[4], _("/Options/Distance Units/(k)pc, AU, miles"));
+  strcpy(options_unit_names[5], _("/Options/Distance Units/(k)pc only"));
+
   /* The following code sets the check buttons in the Options menu to be
      toggled ON by default, matching the initial state of the program.
 
@@ -106,6 +116,9 @@ void set_menu_globals (GtkItemFactory *item_factory)
   for (i = 0; i < OPTIONS_COORD_MENU_SIZE; i++)
     options_coord_widgets[i] =
       gtk_item_factory_get_widget (item_factory, options_coord_names[i]);
+  for (i = 0; i < OPTIONS_UNIT_MENU_SIZE; i++)
+    options_unit_widgets[i] =
+      gtk_item_factory_get_widget (item_factory, options_unit_names[i]);
 
   return;
 }
@@ -170,17 +183,23 @@ void my_gtk_button_bar(GtkWidget **OK_btn, GtkWidget **defaults_btn,
   gtk_widget_show (buttonbox);
   gtk_box_pack_start (GTK_BOX (box), buttonbox, false, false, 0);
 
+  if (OK_btn) {
   *OK_btn = gtk_button_new_from_stock (GTK_STOCK_OK);
   gtk_container_add (GTK_CONTAINER (buttonbox), *OK_btn);
   gtk_widget_show(*OK_btn);
+  }
   
+  if (defaults_btn) {
   *defaults_btn = gtk_button_new_from_stock (GTK_STOCK_REVERT_TO_SAVED);
   gtk_container_add (GTK_CONTAINER (buttonbox), *defaults_btn);
   gtk_widget_show(*defaults_btn);
-  
+  }
+
+  if (cancel_btn) {
   *cancel_btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   gtk_container_add (GTK_CONTAINER (buttonbox), *cancel_btn);
   gtk_widget_show(*cancel_btn);
+  }
   
   return;
 }
@@ -192,12 +211,15 @@ void my_gtk_button_bar(GtkWidget **OK_btn, GtkWidget **defaults_btn,
 void my_gtk_buttons_connect_destroy(GtkWidget *ok, GtkWidget *defaults,
 				    GtkWidget *cancel, GtkWidget *window)
 {
-  g_signal_connect_swapped (G_OBJECT (ok), "clicked",
-    G_CALLBACK (gtk_widget_destroy), G_OBJECT (window));
-  g_signal_connect_swapped (G_OBJECT (defaults), "clicked",
-    G_CALLBACK (gtk_widget_destroy), G_OBJECT (window));
-  g_signal_connect_swapped (G_OBJECT (cancel), "clicked",
-    G_CALLBACK (gtk_widget_destroy), G_OBJECT (window));
+  if (ok)
+    g_signal_connect_swapped (G_OBJECT (ok), "clicked",
+      G_CALLBACK (gtk_widget_destroy), G_OBJECT (window));
+  if (defaults)
+    g_signal_connect_swapped (G_OBJECT (defaults), "clicked",
+      G_CALLBACK (gtk_widget_destroy), G_OBJECT (window));
+  if (cancel)
+    g_signal_connect_swapped (G_OBJECT (cancel), "clicked",
+      G_CALLBACK (gtk_widget_destroy), G_OBJECT (window));
   return;
 }
 
@@ -338,6 +360,95 @@ void cel_coords() { set_coord_item(CELESTIAL, true); }
 void gal_coords() { set_coord_item(GALACTIC, true); }
 
 
+void set_distance_units(const distance_unit units[4], bool redraw)
+{
+  static bool enabled = true;
+
+  if (enabled) {      // wrapper to keep the checkmenuitem toggle from
+    enabled = false;  // causing endless recursive calls of this function
+    int widget_n = -1;
+
+    // Set the appropriate radio menu item to be checked if the desired
+    // unit state corresponds to an existing menu option.
+    if (units[0] == DIST_LY && units[1] == DIST_LY &&
+	units[2] == DIST_AU && units[3] == DIST_KM)	     widget_n = 0;
+    else if (units[0] == DIST_LY && units[1] == DIST_LY &&
+	     units[2] == DIST_AU && units[3] == DIST_MI)     widget_n = 1;
+    else if (units[0] == DIST_LY && units[1] == DIST_LY &&
+	     units[2] == DIST_LY && units[3] == DIST_LY)     widget_n = 2;
+    else if (units[0] == DIST_KPC && units[1] == DIST_PC &&
+	     units[2] == DIST_AU  && units[3] == DIST_KM)    widget_n = 3;
+    else if (units[0] == DIST_KPC && units[1] == DIST_PC &&
+	     units[2] == DIST_AU  && units[3] == DIST_MI)    widget_n = 4;
+    else if (units[0] == DIST_KPC && units[1] == DIST_PC &&
+	     units[2] == DIST_PC  && units[3] == DIST_PC)    widget_n = 5;
+
+    // Now select the menu radio button (if any) that corresponds to the
+    // passed-in set of distance units.
+    if (widget_n >= 0)
+      gtk_check_menu_item_set_active 
+	(GTK_CHECK_MENU_ITEM (options_unit_widgets[widget_n]), true);
+
+    else {
+      // Forcibly de-select all radio buttons in the distance units menu if
+      // the passed-in units do not correspond to any choice available from the
+      // menu (e.g. if the user hand-edited the .starplotrc file).
+      // WARNING: This is not supposed to work in principle: GTK+ does not
+      // have any guaranteed way to deactivate all radio buttons in a group.
+      // In practice it works fine in GTK+ 2.8 and 2.12, but is not guaranteed
+      // to always work correctly.
+      for (unsigned i = 0; i < OPTIONS_UNIT_MENU_SIZE; i++)
+	(GTK_CHECK_MENU_ITEM (options_unit_widgets[i]))->active = false;
+    }
+
+    enabled = true;
+
+    // If the passed-in units are no change from the original units, return.
+    if (units[0] == globals::chart_rules.ChartUnits[0] &&
+	units[1] == globals::chart_rules.ChartUnits[1] &&
+	units[2] == globals::chart_rules.ChartUnits[2] &&
+	units[3] == globals::chart_rules.ChartUnits[3])
+      return;
+
+    // Make the actual change.
+    for (unsigned i = 0; i < 4; i++)
+      globals::chart_rules.ChartUnits[i] = units[i];
+    if (redraw)
+      redraw_all(UNITS_CHANGE);
+  }
+}
+
+void ly_km_units() {
+  const distance_unit units[4] = { DIST_LY, DIST_LY, DIST_AU, DIST_KM };
+  set_distance_units(units, true);
+}
+
+void pc_km_units() {
+  const distance_unit units[4] = { DIST_KPC, DIST_PC, DIST_AU, DIST_KM };
+  set_distance_units(units, true);
+}
+
+void ly_mi_units() {
+  const distance_unit units[4] = { DIST_LY, DIST_LY, DIST_AU, DIST_MI };
+  set_distance_units(units, true);
+}
+
+void pc_mi_units() {
+  const distance_unit units[4] = { DIST_KPC, DIST_PC, DIST_AU, DIST_MI };
+  set_distance_units(units, true);
+}
+
+void ly_units() {
+  const distance_unit units[4] = { DIST_LY, DIST_LY, DIST_LY, DIST_LY };
+  set_distance_units(units, true);
+}
+
+void pc_units() {
+  const distance_unit units[4] = { DIST_KPC, DIST_PC, DIST_PC, DIST_PC };
+  set_distance_units(units, true);
+}
+
+
 /* These callbacks are for the button bar immediately underneath the menu. */
 
 void button_zoom(GtkWidget *w, gpointer zoomfactor)
@@ -372,6 +483,17 @@ void button_tilt(GtkWidget *w, gpointer angle)
   return;
 }
 
+void button_set_earth_orientation(GtkWidget *w, gpointer ignored)
+{
+  if (globals::chart_rules.ChartRadius <
+	globals::chart_rules.ChartLocation.magnitude()) {
+    globals::chart_rules.ChartOrientation
+      = (- globals::chart_rules.ChartLocation).toSpherical();
+    redraw_all(ORIENTATION_CHANGE);
+  }
+  return;
+}
+
 void button_magchange(GtkWidget *w, gpointer magchange)
 {
   double m = *(double *)magchange;
@@ -385,7 +507,7 @@ void button_magchange(GtkWidget *w, gpointer magchange)
 
 void my_gtk_push_buttons (GtkWidget *window, GtkWidget **buttonbar)
 {
-  GtkWidget *vbox, *buttons[9], *pixmapw;
+  GtkWidget *vbox, *buttons[10], *pixmapw;
   GtkTooltips *tooltips;
   GtkStyle *style = gtk_widget_get_style (window);
   GtkStyle *blackbackground = gtk_style_copy (style);
@@ -406,7 +528,7 @@ void my_gtk_push_buttons (GtkWidget *window, GtkWidget **buttonbar)
   blackbackground->bg[GTK_STATE_NORMAL] = blackbackground->black;
   gtk_widget_set_style (*buttonbar, blackbackground);
 
-  for (unsigned int i = 0; i < 9; i++) {
+  for (unsigned int i = 0; i < 10; i++) {
     // this creates the button icons; they are defined in the static
     //  array `xpmdata' in the xpmdata.h header file.
     pixmap = gdk_pixmap_colormap_create_from_xpm_d 
@@ -432,35 +554,39 @@ void my_gtk_push_buttons (GtkWidget *window, GtkWidget **buttonbar)
   g_signal_connect (G_OBJECT (buttons[2]), "clicked",
 		      G_CALLBACK (button_zoom), &zoomoutfactor);
   g_signal_connect (G_OBJECT (buttons[3]), "clicked",
-		      G_CALLBACK (button_rotate), &leftangle);
-  g_signal_connect (G_OBJECT (buttons[4]), "clicked",
-		      G_CALLBACK (button_rotate), &rightangle);
-  g_signal_connect (G_OBJECT (buttons[5]), "clicked",
-		      G_CALLBACK (button_tilt), &northangle);
-  g_signal_connect (G_OBJECT (buttons[6]), "clicked",
-		      G_CALLBACK (button_tilt), &southangle);
-  g_signal_connect (G_OBJECT (buttons[7]), "clicked",
 		      G_CALLBACK (button_magchange), &magup);
-  g_signal_connect (G_OBJECT (buttons[8]), "clicked",
+  g_signal_connect (G_OBJECT (buttons[4]), "clicked",
 		      G_CALLBACK (button_magchange), &magdown);
+  g_signal_connect (G_OBJECT (buttons[5]), "clicked",
+		      G_CALLBACK (button_rotate), &leftangle);
+  g_signal_connect (G_OBJECT (buttons[6]), "clicked",
+		      G_CALLBACK (button_rotate), &rightangle);
+  g_signal_connect (G_OBJECT (buttons[7]), "clicked",
+		      G_CALLBACK (button_tilt), &northangle);
+  g_signal_connect (G_OBJECT (buttons[8]), "clicked",
+		      G_CALLBACK (button_tilt), &southangle);
+  g_signal_connect (G_OBJECT (buttons[9]), "clicked",
+		      G_CALLBACK (button_set_earth_orientation), 0);
 
   // and set some popup help tips
   tooltips = gtk_tooltips_new ();
   gtk_tooltips_set_tip (tooltips, buttons[0], _("Open star database..."), 0);
   gtk_tooltips_set_tip (tooltips, buttons[1], _("Zoom in"), 0);
   gtk_tooltips_set_tip (tooltips, buttons[2], _("Zoom out"), 0);
-  gtk_tooltips_set_tip (tooltips, buttons[3],
-			_("Rotate chart clockwise about its axis"), 0);
-  gtk_tooltips_set_tip (tooltips, buttons[4], 
-			_("Rotate chart counterclockwise about its axis"), 0);
-  gtk_tooltips_set_tip (tooltips, buttons[5], 
-			_("Tilt chart north pole towards you"), 0);
-  gtk_tooltips_set_tip (tooltips, buttons[6], 
-			_("Tilt chart south pole towards you"), 0);
-  gtk_tooltips_set_tip (tooltips, buttons[7], 
+  gtk_tooltips_set_tip (tooltips, buttons[3], 
 			_("Decrease magnitude limit (Show fewer stars)"), 0);
-  gtk_tooltips_set_tip (tooltips, buttons[8], 
+  gtk_tooltips_set_tip (tooltips, buttons[4], 
 			_("Increase magnitude limit (Show more stars)"), 0);
+  gtk_tooltips_set_tip (tooltips, buttons[5],
+			_("Rotate chart clockwise about its axis"), 0);
+  gtk_tooltips_set_tip (tooltips, buttons[6], 
+			_("Rotate chart counterclockwise about its axis"), 0);
+  gtk_tooltips_set_tip (tooltips, buttons[7], 
+			_("Tilt chart north pole towards you"), 0);
+  gtk_tooltips_set_tip (tooltips, buttons[8], 
+			_("Tilt chart south pole towards you"), 0);
+  gtk_tooltips_set_tip (tooltips, buttons[9], 
+			_("Set chart orientation as seen from Earth"), 0);
   return;
 }
 
@@ -472,7 +598,7 @@ void abouthelp()
   GtkTextBuffer *text;
   
   popup = gtk_dialog_new ();
-  gtk_window_set_policy (GTK_WINDOW (popup), false, false, true);
+  gtk_window_set_resizable (GTK_WINDOW (popup), false);
   gtk_window_set_title (GTK_WINDOW (popup), _("About StarPlot"));
   
   topbox = gtk_vbox_new (true, 0);
@@ -485,7 +611,7 @@ void abouthelp()
   gtk_widget_show (frame);
   
   string textstr = string(_("StarPlot version %s\n\n"));
-  textstr += string(_("Copyright %s 2000-2007 Kevin B. McCarty\n"));
+  textstr += string("%s 2000-2008 Kevin B. McCarty\n");
   textstr += string(_("Licensed under the GNU General Public License\n\n"));
   textstr += string(_(
 "For help, please select Help->Documentation from the menu, or \
